Files
TheRock/rocm-libraries/projects/hipsparse/clients/include/testing_spmm_batched_csc.hpp
T
derek.holloway 7502bf64a4
CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
Flatten subprojects into a single base monorepo
2026-09-02 16:34:47 -07:00

521 lines
21 KiB
C++

/* ************************************************************************
* Copyright (C) 2022-2026 Advanced Micro Devices, Inc. All rights Reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
* ************************************************************************ */
#pragma once
#ifndef TESTING_SPMM_BATCHED_CSC_HPP
#define TESTING_SPMM_BATCHED_CSC_HPP
#include "display.hpp"
#include "flops.hpp"
#include "gbyte.hpp"
#include "hipsparse.hpp"
#include "hipsparse_arguments.hpp"
#include "hipsparse_graph.hpp"
#include "hipsparse_test_unique_ptr.hpp"
#include "unit.hpp"
#include "utility.hpp"
#include <hipsparse.h>
#include <string>
#include <typeinfo>
using namespace hipsparse;
using namespace hipsparse_test;
template <typename I, typename J, typename T>
void testing_spmm_batched_csc_bad_arg(const Arguments& argus)
{
#if(!defined(CUDART_VERSION))
int32_t m = 100;
int32_t n = 100;
int32_t k = 100;
int64_t nnz = 100;
float alpha = 0.6;
float beta = 0.2;
size_t safe_size = 100;
hipsparseOperation_t transA = HIPSPARSE_OPERATION_NON_TRANSPOSE;
hipsparseOperation_t transB = HIPSPARSE_OPERATION_NON_TRANSPOSE;
hipsparseOrder_t order = HIPSPARSE_ORDER_COL;
hipsparseIndexBase_t idxBase = HIPSPARSE_INDEX_BASE_ZERO;
hipsparseIndexType_t idxTypeI = HIPSPARSE_INDEX_64I;
hipsparseIndexType_t idxTypeJ = HIPSPARSE_INDEX_32I;
hipDataType dataType = HIP_R_32F;
#if(CUDART_VERSION >= 11003)
hipsparseSpMMAlg_t alg = HIPSPARSE_SPMM_CSR_ALG1;
#else
hipsparseSpMMAlg_t alg = HIPSPARSE_MM_ALG_DEFAULT;
#endif
hipsparseLocalHandle_t handle;
auto dptr_managed
= hipsparse_unique_ptr{device_malloc(sizeof(int64_t) * safe_size), device_free};
auto drow_managed
= hipsparse_unique_ptr{device_malloc(sizeof(int32_t) * safe_size), device_free};
auto dval_managed = hipsparse_unique_ptr{device_malloc(sizeof(float) * safe_size), device_free};
auto dB_managed = hipsparse_unique_ptr{device_malloc(sizeof(float) * safe_size), device_free};
auto dC_managed = hipsparse_unique_ptr{device_malloc(sizeof(float) * safe_size), device_free};
auto dbuf_managed = hipsparse_unique_ptr{device_malloc(sizeof(char) * safe_size), device_free};
int64_t* dptr = (int64_t*)dptr_managed.get();
int32_t* drow = (int32_t*)drow_managed.get();
float* dval = (float*)dval_managed.get();
float* dB = (float*)dB_managed.get();
float* dC = (float*)dC_managed.get();
void* dbuf = (void*)dbuf_managed.get();
// SpMM structures
hipsparseSpMatDescr_t A;
hipsparseDnMatDescr_t B, C;
// Create SpMM structures
verify_hipsparse_status_success(
hipsparseCreateCsc(&A, m, k, nnz, dptr, drow, dval, idxTypeI, idxTypeJ, idxBase, dataType),
"success");
verify_hipsparse_status_success(hipsparseCreateDnMat(&B, k, n, k, dB, dataType, order),
"success");
verify_hipsparse_status_success(hipsparseCreateDnMat(&C, m, n, m, dC, dataType, order),
"success");
int batch_count_A;
int batch_count_B;
int batch_count_C;
int64_t offsets_batch_stride_A;
int64_t rows_values_batch_stride_A;
int64_t batch_stride_B;
int64_t batch_stride_C;
// C_i = A * B_i
batch_count_A = 1;
batch_count_B = 10;
batch_count_C = 5;
offsets_batch_stride_A = 0;
rows_values_batch_stride_A = 0;
batch_stride_B = k * n;
batch_stride_C = m * n;
verify_hipsparse_status_success(
hipsparseCsrSetStridedBatch(
A, batch_count_A, offsets_batch_stride_A, rows_values_batch_stride_A),
"success");
verify_hipsparse_status_success(hipsparseDnMatSetStridedBatch(B, batch_count_B, batch_stride_B),
"success");
verify_hipsparse_status_success(hipsparseDnMatSetStridedBatch(C, batch_count_C, batch_stride_C),
"success");
verify_hipsparse_status_invalid_value(
hipsparseSpMM(handle, transA, transB, &alpha, A, B, &beta, C, dataType, alg, dbuf),
"Error: Combination of strided batch parameters is invald");
// C_i = A_i * B
batch_count_A = 10;
batch_count_B = 1;
batch_count_C = 5;
offsets_batch_stride_A = (k + 1);
rows_values_batch_stride_A = nnz;
batch_stride_B = 0;
batch_stride_C = m * n;
verify_hipsparse_status_success(
hipsparseCsrSetStridedBatch(
A, batch_count_A, offsets_batch_stride_A, rows_values_batch_stride_A),
"success");
verify_hipsparse_status_success(hipsparseDnMatSetStridedBatch(B, batch_count_B, batch_stride_B),
"success");
verify_hipsparse_status_success(hipsparseDnMatSetStridedBatch(C, batch_count_C, batch_stride_C),
"success");
verify_hipsparse_status_invalid_value(
hipsparseSpMM(handle, transA, transB, &alpha, A, B, &beta, C, dataType, alg, dbuf),
"Error: Combination of strided batch parameters is invald");
// C_i = A_i * B_i
batch_count_A = 10;
batch_count_B = 10;
batch_count_C = 5;
offsets_batch_stride_A = (k + 1);
rows_values_batch_stride_A = nnz;
batch_stride_B = k * n;
batch_stride_C = m * n;
verify_hipsparse_status_success(
hipsparseCsrSetStridedBatch(
A, batch_count_A, offsets_batch_stride_A, rows_values_batch_stride_A),
"success");
verify_hipsparse_status_success(hipsparseDnMatSetStridedBatch(B, batch_count_B, batch_stride_B),
"success");
verify_hipsparse_status_success(hipsparseDnMatSetStridedBatch(C, batch_count_C, batch_stride_C),
"success");
verify_hipsparse_status_invalid_value(
hipsparseSpMM(handle, transA, transB, &alpha, A, B, &beta, C, dataType, alg, dbuf),
"Error: Combination of strided batch parameters is invald");
// Destruct
verify_hipsparse_status_success(hipsparseDestroySpMat(A), "success");
verify_hipsparse_status_success(hipsparseDestroyDnMat(B), "success");
verify_hipsparse_status_success(hipsparseDestroyDnMat(C), "success");
#endif
}
template <typename I, typename J, typename T>
void testing_spmm_batched_csc(Arguments argus)
{
#if(!defined(CUDART_VERSION))
J m = argus.M;
J n = argus.N;
J k = argus.K;
T h_alpha = argus.get_alpha<T>();
T h_beta = argus.get_beta<T>();
hipsparseOperation_t transA = argus.transA;
hipsparseOperation_t transB = argus.transB;
hipsparseOrder_t orderB = argus.orderB;
hipsparseOrder_t orderC = argus.orderC;
hipsparseIndexBase_t idx_base = argus.baseA;
J batch_count_A = 1;
J batch_count_B = 3;
J batch_count_C = 3;
#if(CUDART_VERSION >= 11003)
hipsparseSpMMAlg_t alg = HIPSPARSE_SPMM_CSR_ALG1;
#else
hipsparseSpMMAlg_t alg = HIPSPARSE_MM_ALG_DEFAULT;
#endif
std::string filename = argus.filename;
#if(defined(CUDART_VERSION))
if(orderB != orderC || orderB != HIPSPARSE_ORDER_COL)
{
return;
}
#endif
// Index and data type
hipsparseIndexType_t typeI = getIndexType<I>();
hipsparseIndexType_t typeJ = getIndexType<J>();
hipDataType typeT = getDataType<T>();
// hipSPARSE handle
hipsparseLocalHandle_t handle(argus);
// Host structures
std::vector<I> hcsc_col_ptr_temp;
std::vector<J> hcsc_row_ind_temp;
std::vector<T> hcsc_val_temp;
// Initial Data on CPU
srand(12345ULL);
I nnz_A;
CHECK_GENERATE_MATRIX_ERROR(
generate_csr_matrix(filename,
(transA == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? k : m,
(transA == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? m : k,
nnz_A,
hcsc_col_ptr_temp,
hcsc_row_ind_temp,
hcsc_val_temp,
idx_base));
// Redefine sparse matrix values
hipsparseInit<T>(hcsc_val_temp, hcsc_val_temp.size(), 1);
// Some matrix properties
J A_m = (transA == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? m : k;
J A_n = (transA == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? k : m;
J B_m = (transB == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? k : n;
J B_n = (transB == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? n : k;
J C_m = m;
J C_n = n;
int ld_multiplier_B = 1;
int ld_multiplier_C = 1;
int64_t ldb
= (orderB == HIPSPARSE_ORDER_COL)
? ((transB == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? (int64_t(ld_multiplier_B) * k)
: (int64_t(ld_multiplier_B) * n))
: ((transB == HIPSPARSE_OPERATION_NON_TRANSPOSE) ? (int64_t(ld_multiplier_B) * n)
: (int64_t(ld_multiplier_B) * k));
int64_t ldc = (orderC == HIPSPARSE_ORDER_COL) ? (int64_t(ld_multiplier_C) * m)
: (int64_t(ld_multiplier_C) * n);
ldb = std::max(int64_t(1), ldb);
ldc = std::max(int64_t(1), ldc);
int64_t nrowB = (orderB == HIPSPARSE_ORDER_COL) ? ldb : B_m;
int64_t ncolB = (orderB == HIPSPARSE_ORDER_COL) ? B_n : ldb;
int64_t nrowC = (orderC == HIPSPARSE_ORDER_COL) ? ldc : C_m;
int64_t ncolC = (orderC == HIPSPARSE_ORDER_COL) ? C_n : ldc;
int64_t nnz_B = nrowB * ncolB;
int64_t nnz_C = nrowC * ncolC;
int64_t offsets_batch_stride_A = (batch_count_A > 1) ? (A_n + 1) : 0;
int64_t rows_values_batch_stride_A = (batch_count_A > 1) ? nnz_A : 0;
int64_t batch_stride_B = (batch_count_B > 1) ? nnz_B : 0;
int64_t batch_stride_C = (batch_count_C > 1) ? nnz_C : 0;
// Allocate host memory for all batches of A matrix
std::vector<I> hcsc_col_ptr(batch_count_A * (A_n + 1));
std::vector<J> hcsc_row_ind(batch_count_A * nnz_A);
std::vector<T> hcsc_val(batch_count_A * nnz_A);
for(J i = 0; i < batch_count_A; i++)
{
for(J j = 0; j < (A_n + 1); j++)
{
hcsc_col_ptr[(A_n + 1) * i + j] = hcsc_col_ptr_temp[j];
}
for(I j = 0; j < nnz_A; j++)
{
hcsc_row_ind[nnz_A * i + j] = hcsc_row_ind_temp[j];
hcsc_val[nnz_A * i + j] = hcsc_val_temp[j];
}
}
// Allocate host memory for vectors
std::vector<T> hB(batch_count_B * nnz_B);
std::vector<T> hC_1(batch_count_C * nnz_C);
std::vector<T> hC_2(batch_count_C * nnz_C);
std::vector<T> hC_gold(batch_count_C * nnz_C);
hipsparseInit<T>(hB, batch_count_B * nnz_B, 1);
hipsparseInit<T>(hC_1, batch_count_C * nnz_C, 1);
// copy vector is easy in STL; hC_gold = hC: save a copy in hy_gold which will be output of CPU
hC_2 = hC_1;
hC_gold = hC_1;
// allocate memory on device
auto dptr_managed = hipsparse_unique_ptr{device_malloc(sizeof(I) * (A_n + 1)), device_free};
auto drow_managed = hipsparse_unique_ptr{device_malloc(sizeof(J) * nnz_A), device_free};
auto dval_managed = hipsparse_unique_ptr{device_malloc(sizeof(T) * nnz_A), device_free};
auto dB_managed
= hipsparse_unique_ptr{device_malloc(sizeof(T) * batch_count_B * nnz_B), device_free};
auto dC_1_managed
= hipsparse_unique_ptr{device_malloc(sizeof(T) * batch_count_C * nnz_C), device_free};
auto dC_2_managed
= hipsparse_unique_ptr{device_malloc(sizeof(T) * batch_count_C * nnz_C), device_free};
auto d_alpha_managed = hipsparse_unique_ptr{device_malloc(sizeof(T)), device_free};
auto d_beta_managed = hipsparse_unique_ptr{device_malloc(sizeof(T)), device_free};
I* dptr = (I*)dptr_managed.get();
J* drow = (J*)drow_managed.get();
T* dval = (T*)dval_managed.get();
T* dB = (T*)dB_managed.get();
T* dC_1 = (T*)dC_1_managed.get();
T* dC_2 = (T*)dC_2_managed.get();
T* d_alpha = (T*)d_alpha_managed.get();
T* d_beta = (T*)d_beta_managed.get();
// copy data from CPU to device
CHECK_HIP_ERROR(
hipMemcpy(dptr, hcsc_col_ptr.data(), sizeof(I) * (A_n + 1), hipMemcpyHostToDevice));
CHECK_HIP_ERROR(hipMemcpy(drow, hcsc_row_ind.data(), sizeof(J) * nnz_A, hipMemcpyHostToDevice));
CHECK_HIP_ERROR(hipMemcpy(dval, hcsc_val.data(), sizeof(T) * nnz_A, hipMemcpyHostToDevice));
CHECK_HIP_ERROR(
hipMemcpy(dB, hB.data(), sizeof(T) * batch_count_B * nnz_B, hipMemcpyHostToDevice));
CHECK_HIP_ERROR(
hipMemcpy(dC_1, hC_1.data(), sizeof(T) * batch_count_C * nnz_C, hipMemcpyHostToDevice));
CHECK_HIP_ERROR(
hipMemcpy(dC_2, hC_2.data(), sizeof(T) * batch_count_C * nnz_C, hipMemcpyHostToDevice));
CHECK_HIP_ERROR(hipMemcpy(d_alpha, &h_alpha, sizeof(T), hipMemcpyHostToDevice));
CHECK_HIP_ERROR(hipMemcpy(d_beta, &h_beta, sizeof(T), hipMemcpyHostToDevice));
// Create matrices
hipsparseSpMatDescr_t A;
CHECK_HIPSPARSE_ERROR(
hipsparseCreateCsc(&A, A_m, A_n, nnz_A, dptr, drow, dval, typeI, typeJ, idx_base, typeT));
// Create dense matrices
hipsparseDnMatDescr_t B, C1, C2;
CHECK_HIPSPARSE_ERROR(hipsparseCreateDnMat(&B, B_m, B_n, ldb, dB, typeT, orderB));
CHECK_HIPSPARSE_ERROR(hipsparseCreateDnMat(&C1, C_m, C_n, ldc, dC_1, typeT, orderC));
CHECK_HIPSPARSE_ERROR(hipsparseCreateDnMat(&C2, C_m, C_n, ldc, dC_2, typeT, orderC));
CHECK_HIPSPARSE_ERROR(hipsparseCsrSetStridedBatch(
A, batch_count_A, offsets_batch_stride_A, rows_values_batch_stride_A));
CHECK_HIPSPARSE_ERROR(hipsparseDnMatSetStridedBatch(B, batch_count_B, batch_stride_B));
CHECK_HIPSPARSE_ERROR(hipsparseDnMatSetStridedBatch(C1, batch_count_C, batch_stride_C));
CHECK_HIPSPARSE_ERROR(hipsparseDnMatSetStridedBatch(C2, batch_count_C, batch_stride_C));
// Query SpMM buffer
size_t bufferSize;
CHECK_HIPSPARSE_ERROR(testing::hipsparseSpMM_bufferSize(
handle, transA, transB, &h_alpha, A, B, &h_beta, C1, typeT, alg, &bufferSize));
#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 11021)
//When using cusparse backend, cant pass nullptr for buffer to preprocess
if(bufferSize == 0)
{
bufferSize = 4;
}
#endif
void* buffer;
CHECK_HIP_ERROR(hipMalloc(&buffer, bufferSize));
if(argus.call_preprocess)
{
// HIPSPARSE pointer mode host
#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 11021)
CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_HOST));
CHECK_HIPSPARSE_ERROR(testing::hipsparseSpMM_preprocess(
handle, transA, transB, &h_alpha, A, B, &h_beta, C1, typeT, alg, buffer));
#endif
// HIPSPARSE pointer mode device
#if(!defined(CUDART_VERSION) || CUDART_VERSION >= 11021)
CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_DEVICE));
CHECK_HIPSPARSE_ERROR(testing::hipsparseSpMM_preprocess(
handle, transA, transB, d_alpha, A, B, d_beta, C2, typeT, alg, buffer));
#endif
}
if(argus.unit_check)
{
CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_HOST));
CHECK_HIPSPARSE_ERROR(testing::hipsparseSpMM(
handle, transA, transB, &h_alpha, A, B, &h_beta, C1, typeT, alg, buffer));
CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_DEVICE));
CHECK_HIPSPARSE_ERROR(testing::hipsparseSpMM(
handle, transA, transB, d_alpha, A, B, d_beta, C2, typeT, alg, buffer));
// copy output from device to CPU
CHECK_HIP_ERROR(
hipMemcpy(hC_1.data(), dC_1, sizeof(T) * batch_count_C * nnz_C, hipMemcpyDeviceToHost));
CHECK_HIP_ERROR(
hipMemcpy(hC_2.data(), dC_2, sizeof(T) * batch_count_C * nnz_C, hipMemcpyDeviceToHost));
// CPU
host_cscmm_batched(A_m,
n,
A_n,
batch_count_A,
(I)offsets_batch_stride_A,
(I)rows_values_batch_stride_A,
transA,
transB,
h_alpha,
hcsc_col_ptr.data(),
hcsc_row_ind.data(),
hcsc_val.data(),
hB.data(),
(J)ldb,
batch_count_B,
(I)batch_stride_B,
orderB,
h_beta,
hC_gold.data(),
(J)ldc,
batch_count_C,
(I)batch_stride_C,
orderC,
idx_base);
unit_check_near(1, batch_count_C * nnz_C, 1, hC_gold.data(), hC_1.data());
unit_check_near(1, batch_count_C * nnz_C, 1, hC_gold.data(), hC_2.data());
}
if(argus.timing)
{
int number_cold_calls = 2;
int number_hot_calls = argus.iters;
CHECK_HIPSPARSE_ERROR(hipsparseSetPointerMode(handle, HIPSPARSE_POINTER_MODE_HOST));
// Warm up
for(int iter = 0; iter < number_cold_calls; ++iter)
{
CHECK_HIPSPARSE_ERROR(testing::hipsparseSpMM(
handle, transA, transB, &h_alpha, A, B, &h_beta, C1, typeT, alg, buffer));
}
double gpu_time_used = get_time_us();
// Performance run
for(int iter = 0; iter < number_hot_calls; ++iter)
{
CHECK_HIPSPARSE_ERROR(testing::hipsparseSpMM(
handle, transA, transB, &h_alpha, A, B, &h_beta, C1, typeT, alg, buffer));
}
gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
double gflop_count
= batch_count_C
* spmm_gflop_count(n, nnz_A, (I)C_m * (I)C_n, h_beta != make_DataType<T>(0));
double gpu_gflops = get_gpu_gflops(gpu_time_used, gflop_count);
double gbyte_count = cscmm_batched_gbyte_count<T>(A_n,
nnz_A,
(I)B_m * (I)B_n,
(I)C_m * (I)C_n,
batch_count_A,
batch_count_B,
batch_count_C,
h_beta != make_DataType<T>(0));
double gpu_gbyte = get_gpu_gbyte(gpu_time_used, gbyte_count);
display_timing_info(display_key_t::M,
m,
display_key_t::N,
n,
display_key_t::K,
k,
display_key_t::nnzA,
nnz_A,
display_key_t::batch_countA,
batch_count_A,
display_key_t::batch_countB,
batch_count_B,
display_key_t::batch_countC,
batch_count_C,
display_key_t::alpha,
h_alpha,
display_key_t::beta,
h_beta,
display_key_t::algorithm,
hipsparse_spmmalg2string(alg),
display_key_t::gflops,
gpu_gflops,
display_key_t::bandwidth,
gpu_gbyte,
display_key_t::time_ms,
get_gpu_time_msec(gpu_time_used));
}
CHECK_HIP_ERROR(hipFree(buffer));
CHECK_HIPSPARSE_ERROR(hipsparseDestroySpMat(A));
CHECK_HIPSPARSE_ERROR(hipsparseDestroyDnMat(B));
CHECK_HIPSPARSE_ERROR(hipsparseDestroyDnMat(C1));
CHECK_HIPSPARSE_ERROR(hipsparseDestroyDnMat(C2));
#endif
}
#endif // TESTING_SPMM_BATCHED_CSC_HPP